@@ -5,15 +5,14 @@ import Data.Conduit
 import qualified Data.Conduit.List as CL
 import Data.Conduit.Filesystem
 import Data.List (sort, isSuffixOf)
-import Control.Monad.Trans.Resource (runResourceT)
 
 spec :: Spec
 spec = describe "Data.Conduit.Filesystem" $ do
     it "sourceDirectory" $ do
-        res <- runResourceT
+        res <- runConduitRes
              $ sourceDirectory "test/filesystem"
-             $$ CL.filter (not . (".swp" `isSuffixOf`))
-             =$ CL.consume
+            .| CL.filter (not . (".swp" `isSuffixOf`))
+            .| CL.consume
         sort res `shouldBe`
             [ "test/filesystem/bar.txt"
             , "test/filesystem/baz.txt"
@@ -21,14 +20,14 @@ spec = describe "Data.Conduit.Filesystem" $ do
             , "test/filesystem/foo.txt"
             ]
     it "sourceDirectoryDeep" $ do
-        res1 <- runResourceT
+        res1 <- runConduitRes
               $ sourceDirectoryDeep False "test/filesystem"
-              $$ CL.filter (not . (".swp" `isSuffixOf`))
-              =$ CL.consume
-        res2 <- runResourceT
+             .| CL.filter (not . (".swp" `isSuffixOf`))
+             .| CL.consume
+        res2 <- runConduitRes
               $ sourceDirectoryDeep True "test/filesystem"
-              $$ CL.filter (not . (".swp" `isSuffixOf`))
-              =$ CL.consume
+             .| CL.filter (not . (".swp" `isSuffixOf`))
+             .| CL.consume
         sort res1 `shouldBe`
             [ "test/filesystem/bar.txt"
             , "test/filesystem/baz.txt"
